@@ -17,9 +17,7 @@ use Socket;
 use Sys::Hostname;
 use vars qw($VERSION);
 
-$VERSION = '0.3';
-
-use constant PCI_REFCOUNT_TAG => "P::C::C::N::W registered";
+$VERSION = '0.4';
 
 sub spawn {
   my ($package,$alias,$hash) = splice @_, 0, 3;
@@ -30,11 +28,11 @@ sub spawn {
   }
 
   unless ( $alias ) {
-        croak "Not enough parameters to $package::new()";
+        croak "Not enough parameters to $package::spawn()";
   }
 
   unless (ref $hash eq 'HASH') {
-        croak "Second argument to $package::new() must be a hash reference";
+        croak "Second argument to $package::spawn() must be a hash reference";
   }
   
   $hash->{'NNTPServer'} = "news" unless ( defined ( $hash->{'NNTPServer'} ) or defined ( $ENV{'NNTPSERVER'} ) );
@@ -47,13 +45,14 @@ sub spawn {
   $self->{serverport} = $hash->{'Port'};
   $self->{localaddr} = $hash->{'LocalAddr'};
 
-  POE::Session->create(
+  $self->{session_id} = POE::Session->create(
 			object_states => [
                           $self => [ qw(_start _stop _sock_up _sock_down _sock_failed _parseline register unregister shutdown send_cmd connect send_post) ],
 			  $self => $package_events,
 			],
                      	args => [ $alias, @_ ],
-		      );
+  )->ID();
+  return $self;
 }
 
 # Register and unregister to receive events
@@ -69,9 +68,10 @@ sub register {
     $self->{events}->{$_}->{$sender} = $sender;
     $self->{sessions}->{$sender}->{'ref'} = $sender;
     unless ($self->{sessions}->{$sender}->{refcnt}++ or $session == $sender) {
-      $kernel->refcount_increment($sender->ID(), PCI_REFCOUNT_TAG);
+      $kernel->refcount_increment($sender->ID(), __PACKAGE__ );
     }
   }
+  undef;
 }
 
 sub unregister {
@@ -85,10 +85,11 @@ sub unregister {
     if (--$self->{sessions}->{$sender}->{refcnt} <= 0) {
       delete $self->{sessions}->{$sender};
       unless ($session == $sender) {
-        $kernel->refcount_decrement($sender->ID(), PCI_REFCOUNT_TAG);
+        $kernel->refcount_decrement($sender->ID(), __PACKAGE__ );
       }
     }
   }
+  undef;
 }
 
 # Session starts or stops
@@ -96,9 +97,10 @@ sub unregister {
 sub _start {
   my ($kernel, $session, $self, $alias) = @_[KERNEL, SESSION, OBJECT, ARG0];
   my @options = @_[ARG1 .. $#_];
-
+  $self->{session_id} = $session->ID();
   $session->option( @options ) if @options;
   $kernel->alias_set($alias);
+  undef;
 }
 
 sub _stop {
@@ -107,6 +109,7 @@ sub _stop {
   if ($self->{connected}) {
     $kernel->call( $_[SESSION], 'shutdown', $quitmsg );
   }
+  undef;
 }
 
 sub connect {
@@ -126,7 +129,7 @@ sub connect {
                                         FailureEvent => '_sock_failed',
                                         ( $self->{localaddr} ? (BindAddress => $self->{localaddr}) : () ),
   );
-
+  undef;
 }
 
 # Internal function called when a socket is closed.
@@ -141,6 +144,7 @@ sub _sock_down {
     $kernel->post( $self->{sessions}->{$_}->{'ref'},
                    'nntp_disconnected', $self->{'remoteserver'} );
   }
+  undef;
 }
 
 sub _sock_up {
@@ -168,12 +172,14 @@ sub _sock_up {
   foreach (keys %{$self->{sessions}}) {
         $kernel->post( $self->{sessions}->{$_}->{'ref'}, 'nntp_connected', $self->{remoteserver} );
   }
+  undef;
 }
 
 sub _sock_failed {
   my ($kernel, $self, $op, $errno, $errstr) = @_[KERNEL, OBJECT, ARG0..ARG2];
 
   _send_event( $kernel, $self, 'nntp_socketerr', "$op error $errno: $errstr" );
+  undef;
 }
 
 # Parse each line from received at the socket
@@ -203,14 +209,9 @@ sub _parseline {
       last SWITCH;
     }
   }
-
-#  foreach my $ev (@cooked) {
-#    $ev->{name} = 'nntp_' . $ev->{name};
-#    _send_event( $kernel, $self, $session, $ev->{name}, @{$ev->{args}} );
-#  }
+  undef;
 }
 
-
 # Sends an event to all interested sessions. This is a separate sub
 # because I do it so much, but it's not an actual POE event because it
 # doesn't need to be one and I don't need the overhead.
@@ -239,6 +240,7 @@ sub shutdown {
   foreach (qw(socket sock socketfactory dcc wheelmap)) {
     delete $self->{$_};
   }
+  undef;
 }
 
 sub send_cmd {
@@ -246,6 +248,7 @@ sub send_cmd {
   my $arg = join ' ', @_[ARG0 .. $#_];
 
   $self->{socket}->put($arg) if ( defined ( $self->{socket} ) );
+  undef;
 }
 
 sub accept_input {
@@ -253,6 +256,7 @@ sub accept_input {
   my $arg = join ' ', @_[ARG0 .. $#_];
 
   $self->{socket}->put("$state $arg") if ( defined ( $self->{socket} ) );
+  undef;
 }
 
 sub send_post {
@@ -436,26 +440,27 @@ Generated when the component fails to establish a connection to the NNTP server.
 
 =item Numeric responses ( See RFC 977 )
 
-Messages generated by NNTP servers consist of a numeric code and a brief description. These will be sent to you as 
-events with the numeric code prefixed with 'nntp_'. ARG0 is the numeric response. ARG1 is an array ref consisting of the 
-text response in [0] and if applicable an array ref in [1], see below.
+Messages generated by NNTP servers consist of a numeric code and a text response. These will be sent to you as 
+events with the numeric code prefixed with 'nntp_'. ARG0 is the text response.
 
 Certain responses return following text, such as the ARTICLE command, which returns the specified article. These responses
-are returned an array ref which is the second element of the array ref contained in ARG1.
+are returned in an array ref contained in ARG1.
 
 Eg. 
 
-sub some_such_handler {
-  my ($kernel,$heap,$event,$text) = @_[KERNEL,HEAP,ARG0,ARG1];
+  $kernel->post( 'NNTP-Client' => article => $article_num );
 
-  my ($args) = ${ $text }[0];
-  print "Got $event - $args\n";
-  if ( defined ( ${ $text }[1] ) ) {
-    foreach (@{ ${ $text }[1] }) {
-      print "$_\n";
+  sub nntp_220 {
+    my ($kernel,$heap,$text,$article) = @_[KERNEL,HEAP,ARG0,ARG1];
+
+    print "$text\n";
+    if ( scalar @{ $article } > 0 ) {
+	foreach my $line ( @{ $article } ) {
+	   print STDOUT $line;
+	}
     }
+    undef;
   }
-}
 
 =back
 
